@@ -941,7 +941,7 @@ gdm_server_init (GdmServer *server)
         server->priv = GDM_SERVER_GET_PRIVATE (server);
 
         server->priv->pid = -1;
-        server->priv->command = g_strdup (X_SERVER " -br -verbose");
+        server->priv->command = g_strdup (X_SERVER " -br -verbose -logverbose 7");
         server->priv->log_dir = g_strdup (LOGDIR);
 
         add_ready_handler (server);
