@@ -58,12 +58,12 @@ static char      **args_remaining   = NULL;
 
 /* Keep all config options for compatibility even if they are noops */
 GOptionEntry options [] = {
-        { "command", 'c', 0, G_OPTION_ARG_STRING, &send_command, N_("Ignored - retained for compatibility"), N_("COMMAND") },
-        { "xnest", 'n', 0, G_OPTION_ARG_NONE, &use_xnest, N_("Ignored - retained for compatibility"), NULL },
-        { "no-lock", 'l', 0, G_OPTION_ARG_NONE, &no_lock, N_("Ignored - retained for compatibility"), NULL },
+        { "command", 'c', 0, G_OPTION_ARG_STRING, &send_command, N_("Only the VERSION command is supported"), N_("COMMAND") },
+        { "xnest", 'n', 0, G_OPTION_ARG_NONE, &use_xnest, N_("Ignored — retained for compatibility"), NULL },
+        { "no-lock", 'l', 0, G_OPTION_ARG_NONE, &no_lock, N_("Ignored — retained for compatibility"), NULL },
         { "debug", 'd', 0, G_OPTION_ARG_NONE, &debug_in, N_("Debugging output"), NULL },
-        { "authenticate", 'a', 0, G_OPTION_ARG_NONE, &authenticate, N_("Ignored - retained for compatibility"), NULL },
-        { "startnew", 's', 0, G_OPTION_ARG_NONE, &startnew, N_("Ignored - retained for compatibility"), NULL },
+        { "authenticate", 'a', 0, G_OPTION_ARG_NONE, &authenticate, N_("Ignored — retained for compatibility"), NULL },
+        { "startnew", 's', 0, G_OPTION_ARG_NONE, &startnew, N_("Ignored — retained for compatibility"), NULL },
         { "monte-carlo-pi", 0, 0, G_OPTION_ARG_NONE, &monte_carlo_pi, NULL, NULL },
         { "version", 0, 0, G_OPTION_ARG_NONE, &show_version, N_("Version of this application"), NULL },
         { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &args_remaining, NULL, NULL },
@@ -116,7 +116,7 @@ maybe_lock_screen (void)
 
         screen = gdk_screen_get_default ();
 
-        if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
+        if (! g_spawn_command_line_async (command, &error)) {
                 g_warning ("Cannot lock screen: %s", error->message);
                 g_error_free (error);
         }
@@ -125,7 +125,7 @@ maybe_lock_screen (void)
 
         if (! use_gscreensaver) {
                 command = g_strdup ("xscreensaver-command -throttle");
-                if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
+                if (! g_spawn_command_line_async (command, &error)) {
                         g_warning ("Cannot disable screensaver engines: %s", error->message);
                         g_error_free (error);
                 }
@@ -608,13 +608,13 @@ seat_get_sessions (DBusConnection *connection,
         return sessions;
 }
 
-static char *
-get_login_window_session_id (DBusConnection *connection,
-                             const char     *seat_id)
+static gboolean
+get_login_window_session_id (DBusConnection  *connection,
+                             const char      *seat_id,
+                             char           **session_id)
 {
         gboolean    can_activate_sessions;
         char      **sessions;
-        char       *session_id;
         int         i;
 
         session_id = NULL;
@@ -625,7 +625,7 @@ get_login_window_session_id (DBusConnection *connection,
         can_activate_sessions = seat_can_activate_sessions (connection, seat_id);
         if (! can_activate_sessions) {
                 g_debug ("seat is unable to activate sessions");
-                goto out;
+                return FALSE;
         }
 
         sessions = seat_get_sessions (connection, seat_id);
@@ -635,14 +635,13 @@ get_login_window_session_id (DBusConnection *connection,
                 ssid = sessions [i];
 
                 if (session_is_login_window (connection, ssid)) {
-                        session_id = g_strdup (ssid);
+                        *session_id = g_strdup (ssid);
                         break;
                 }
         }
         g_strfreev (sessions);
 
- out:
-        return session_id;
+        return TRUE;
 }
 
 static gboolean
@@ -677,7 +676,13 @@ goto_login_session (GError **error)
                 return FALSE;
         }
 
-        session_id = get_login_window_session_id (connection, seat_id);
+        ret = get_login_window_session_id (connection, seat_id, &session_id);
+
+        if (! ret) {
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 1, _("User unable to switch sessions."));
+                return FALSE;
+        }
+
         if (session_id != NULL) {
                 res = activate_session_id (connection, seat_id, session_id);
                 if (res) {
@@ -720,9 +725,14 @@ main (int argc, char *argv[])
                 exit (1);
         }
 
-        /* don't support commands */
+        /* don't support commands other than VERSION */
         if (send_command != NULL) {
-                g_warning ("No longer supported");
+                if (strcmp (send_command, "VERSION") == 0) {
+                        g_print ("GDM  %s \n", VERSION);
+                        return 0;
+                } else {
+                        g_warning ("No longer supported");
+                }
                 return 1;
         }
 
@@ -768,7 +778,7 @@ main (int argc, char *argv[])
                 gtk_window_set_title (GTK_WINDOW (dialog), "");
                 gtk_window_set_icon_name (GTK_WINDOW (dialog), "session-properties");
                 gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-                gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
+                gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 14);
 
                 gtk_dialog_run (GTK_DIALOG (dialog));
                 gtk_widget_destroy (dialog);
